<?php
/**
 * Twenty Nineteen functions and definitions
 *
 * @link https://developer.wordpress.org/themes/basics/theme-functions/
 *
 * @package WordPress
 * @subpackage Twenty_Nineteen
 * @since Twenty Nineteen 1.0
 */

/**
 * Twenty Nineteen only works in WordPress 4.7 or later.
 */
if ( version_compare( $GLOBALS['wp_version'], '4.7', '<' ) ) {
	require get_template_directory() . '/inc/back-compat.php';
	return;
}

if ( ! function_exists( 'twentynineteen_setup' ) ) :
	/**
	 * Sets up theme defaults and registers support for various WordPress features.
	 *
	 * Note that this function is hooked into the after_setup_theme hook, which
	 * runs before the init hook. The init hook is too late for some features, such
	 * as indicating support for post thumbnails.
	 */
	function twentynineteen_setup() {
		/*
		 * Make theme available for translation.
		 * Translations can be filed in the /languages/ directory.
		 * If you're building a theme based on Twenty Nineteen, use a find and replace
		 * to change 'twentynineteen' to the name of your theme in all the template files.
		 */
		load_theme_textdomain( 'twentynineteen', get_template_directory() . '/languages' );

		// Add default posts and comments RSS feed links to head.
		add_theme_support( 'automatic-feed-links' );

		/*
		 * Let WordPress manage the document title.
		 * By adding theme support, we declare that this theme does not use a
		 * hard-coded <title> tag in the document head, and expect WordPress to
		 * provide it for us.
		 */
		add_theme_support( 'title-tag' );

		/*
		 * Enable support for Post Thumbnails on posts and pages.
		 *
		 * @link https://developer.wordpress.org/themes/functionality/featured-images-post-thumbnails/
		 */
		add_theme_support( 'post-thumbnails' );
		set_post_thumbnail_size( 1568, 9999 );

		// This theme uses wp_nav_menu() in two locations.
		register_nav_menus(
			array(
				'menu-1' => __( 'Primary', 'twentynineteen' ),
				'footer' => __( 'Footer Menu', 'twentynineteen' ),
				'social' => __( 'Social Links Menu', 'twentynineteen' ),
			)
		);

		/*
		 * Switch default core markup for search form, comment form, and comments
		 * to output valid HTML5.
		 */
		add_theme_support(
			'html5',
			array(
				'search-form',
				'comment-form',
				'comment-list',
				'gallery',
				'caption',
				'script',
				'style',
			)
		);
		
		/**
		 * Add support for core custom logo.
		 *
		 * @link https://codex.wordpress.org/Theme_Logo
		 */
		add_theme_support(
			'custom-logo',
			array(
			
			)
		);

		// Add theme support for selective refresh for widgets.
		add_theme_support( 'customize-selective-refresh-widgets' );

		// Add support for Block Styles.
		add_theme_support( 'wp-block-styles' );

		// Add support for full and wide align images.
		add_theme_support( 'align-wide' );

		// Add support for editor styles.
		add_theme_support( 'editor-styles' );

		// Enqueue editor styles.
		add_editor_style( 'style-editor.css' );

		// Add custom editor font sizes.
		add_theme_support(
			'editor-font-sizes',
			array(
				array(
					'name'      => __( 'Small', 'twentynineteen' ),
					'shortName' => __( 'S', 'twentynineteen' ),
					'size'      => 19.5,
					'slug'      => 'small',
				),
				array(
					'name'      => __( 'Normal', 'twentynineteen' ),
					'shortName' => __( 'M', 'twentynineteen' ),
					'size'      => 22,
					'slug'      => 'normal',
				),
				array(
					'name'      => __( 'Large', 'twentynineteen' ),
					'shortName' => __( 'L', 'twentynineteen' ),
					'size'      => 36.5,
					'slug'      => 'large',
				),
				array(
					'name'      => __( 'Huge', 'twentynineteen' ),
					'shortName' => __( 'XL', 'twentynineteen' ),
					'size'      => 49.5,
					'slug'      => 'huge',
				),
			)
		);

		

		// Add support for responsive embedded content.
		add_theme_support( 'responsive-embeds' );
	}
endif;
add_action( 'after_setup_theme', 'twentynineteen_setup' );

/**
 * Register widget area.
 *
 * @link https://developer.wordpress.org/themes/functionality/sidebars/#registering-a-sidebar
 */
function twentynineteen_widgets_init() {

	register_sidebar(
		array(
			'name'          => __( 'Footer', 'twentynineteen' ),
			'id'            => 'sidebar-1',
			'description'   => __( 'Add widgets here to appear in your footer.', 'twentynineteen' ),
			'before_widget' => '<section id="%1$s" class="widget %2$s">',
			'after_widget'  => '</section>',
			'before_title'  => '<h2 class="widget-title">',
			'after_title'   => '</h2>',
		)
	);

}
add_action( 'widgets_init', 'twentynineteen_widgets_init' );

/**
 * Set the content width in pixels, based on the theme's design and stylesheet.
 *
 * Priority 0 to make it available to lower priority callbacks.
 *
 * @global int $content_width Content width.
 */
function twentynineteen_content_width() {
	// This variable is intended to be overruled from themes.
	// Open WPCS issue: {@link https://github.com/WordPress-Coding-Standards/WordPress-Coding-Standards/issues/1043}.
	// phpcs:ignore WordPress.NamingConventions.PrefixAllGlobals.NonPrefixedVariableFound
	$GLOBALS['content_width'] = apply_filters( 'twentynineteen_content_width', 640 );
}
add_action( 'after_setup_theme', 'twentynineteen_content_width', 0 );

/**
 * Enqueue scripts and styles.
 */
function twentynineteen_scripts() {
	
	//wp_enqueue_script( 'twentynineteen-priority-menu', get_theme_file_uri( '/js/analytics.js.download' ), array(), '20181214', true );
//	wp_enqueue_script( 'twentynineteen-touch-navigation', get_theme_file_uri( '/js/cbbe2bf80ef9ede874dcb45038560437.js.download' ), array('jquery'), '20181231', true );
	//wp_enqueue_script( 'twentynineteen-touch-navigation1', get_theme_file_uri( '/js/lazyload.min.js.download' ), array(), '2018123xx1', true );
	
	wp_enqueue_style( 'twentynineteen-style', get_stylesheet_uri(), array(), wp_get_theme()->get( 'Version' ) );


}
add_action( 'wp_enqueue_scripts', 'twentynineteen_scripts' );

require get_template_directory() . '/functions/header-section.php';
require get_template_directory() . '/functions/home-section.php';
require get_template_directory() . '/functions/about-section.php';
require get_template_directory() . '/functions/service-section.php';
require get_template_directory() . '/functions/we-also-do-section.php';
require get_template_directory() . '/functions/dropshippeur-section.php';
require get_template_directory() . '/functions/testimonials-section.php';
require get_template_directory() . '/functions/features-section.php';
require get_template_directory() . '/functions/business-deal-section.php';
require get_template_directory() . '/functions/about-us-footer-section.php';
require get_template_directory() . '/functions/footer-cta.php';
require get_template_directory() . '/functions/contact-us-section.php';
require get_template_directory() . '/functions/footer-section.php';


add_filter( 'nav_menu_link_attributes', function($atts) {
        $atts['class'] = "raven-menu-item raven-link-item";
        return $atts;
}, 100, 1 );

function helpwp_custom_logo_output( $html ) {
	$html = str_replace('custom-logo', 'raven-site-logo-desktop raven-site-logo-tablet raven-site-logo-mobile', $html );
	return $html;
}
add_filter('get_custom_logo', 'helpwp_custom_logo_output', 10);


add_action('after_setup_theme', 'remove_admin_bar');
 
function remove_admin_bar() {
if (!current_user_can('administrator') && !is_admin()) {
  show_admin_bar(false);
}
}

add_filter( 'user_registration_account_menu_items', 'ur_custom_my_messages3', 10, 1 );
function ur_custom_my_messages3( $items ) {
    $items['adresse-de-facturation'] = __( 'Adresse de facturation', 'user-registration' );
    return $items;
}



add_filter( 'user_registration_account_menu_items', 'ur_custom_my_messages', 10, 1 );
function ur_custom_my_messages( $items ) {
    $items['message-dashboard'] = __( 'Mes messages', 'user-registration' );
    return $items;
}

add_filter( 'user_registration_account_menu_items', 'ur_custom_shopify_integrations', 10, 1 );
function ur_custom_shopify_integrations( $items ) {
    $items['shopify-integration'] = __( 'Shopify Intégration', 'user-registration' );
    return $items;
}



add_filter( 'user_registration_account_menu_items', 'ur_custom_woo_integrations', 10, 1 );
function ur_custom_woo_integrations( $items ) {
    $items['woo-integration'] = __( 'Woo Intégration', 'user-registration' );
    return $items;
}


add_filter( 'user_registration_account_menu_items', 'ur_custom_menu_items', 10, 1 );
function ur_custom_menu_items( $items ) {
    $items['demande-de-sourcing'] = __( 'Demande de sourcing', 'user-registration' );
    return $items;
}




add_filter( 'user_registration_account_menu_items', 'ur_custom_my_messages1', 10, 1 );
function ur_custom_my_messages1( $items ) {
    $items['add-tickets'] = __( 'Demande SAV', 'user-registration' );
    return $items;
}


add_filter( 'user_registration_account_menu_items', 'ur_custom_my_messages2', 10, 1 );
function ur_custom_my_messages2( $items ) {
    $items['show-tickets'] = __( 'Historique SAV', 'user-registration' );
    return $items;
}




function norahme_user_shortcode() {
	ob_start();
	
	$user = wp_get_current_user();
 $roles = ( array ) $user->roles;
 if($roles[0] == 'subscriber'){
     
     $member = 'Membre';
 }else {
     $member = $roles[0];
 }
 

 
 
//echo $image[0];
 
	?> <div class="sidebar-header"> <a href="<?php echo site_url(); ?>"><img src="<?php echo get_theme_mod('footer_logo')?>" alt="Norahme" /></a>
    <div id="close-sidebar"><i class="fa fa-chevron-left"></i></div>
  </div>
  <div class="sidebar-admin">
    <div class="user-pic"> <?php echo get_avatar( $user->ID, 50 ); ?> </div>
    <div class="user-info"> <span class="user-name"><?php echo $user->user_login;?></span> <span class="user-role">@<?php echo $member;?> </span> </div>
  </div><?php
	return ob_get_clean();
}


add_shortcode( 'norahme_user', 'norahme_user_shortcode' );



function wpfi_change_text( $translated_text ) {
	if ( $translated_text == 'Username or email address' ) {
		$translated_text = "Nom d'utilisateur ou adresse e-mail";
	}
	
		if ( $translated_text == 'Password' ) {
		$translated_text = "Mots de passe";
    	}
    	
    	if ( $translated_text == 'Login' ) {
		$translated_text = "Se connecter";
    	}
    	
    	if ( $translated_text == 'Remember me' ) {
		$translated_text = "Se souvenir de moi";
    	}
    	if ( $translated_text == 'Lost your password?' ) {
		$translated_text = "Mot de passe perdu?";
    	}
    	if ( $translated_text == 'Sign out' ) {
		$translated_text = "Déconnexion";
    	}
    	
    	
	return $translated_text;
}
add_filter( 'gettext', 'wpfi_change_text', 99 );



add_action('wp_logout','ps_redirect_after_logout');
function ps_redirect_after_logout(){
         wp_redirect( '/sidentifier/' );
         exit();
}



// Print javascript code in the footer of the site to trigger the popup upon form submission
add_action('wp_footer', 'add_popups_trigger_code',150 );
function add_popups_trigger_code(){
    // show code on certain page, comment out to print code on every page
    if( ! is_page('60') )
        return;
?>
<script>  
// Create a new object for custom validation of a custom field.
var myPopupsSubmitController = Marionette.Object.extend( {
  
  initialize: function() {
    this.listenTo( Backbone.Radio.channel( 'forms' ), 'submit:response', this.actionSubmit );
  },

  actionSubmit: function( response ) {
    //insert popup ID here
    SPU.show('761');
  },

});

// On Document Ready...
jQuery( document ).ready( function( $ ) {
SPU.show('761');
    // Instantiate our custom field's controller, defined above.
   
});
</script>  
<?php
}

// Add this to functions.php
add_shortcode( 'loginerror', 'myErrorShortcode' );

function myErrorShortcode() {

    if( isset( $_GET['login'] ) && $_GET['login'] == 'failed' ) {
        
        //$err_codes = $errors->get_error_codes();
        //print_r($err_codes);

        // Start "recording"
        ob_start(); ?>
        
        
        <ul class="user-registration-error">
			<li><strong>ERROR:</strong>Le nom d'utilisateur / courriel ou mot de passe est incorrect</li>
	</ul>
	 <?php

        // Return result
        return ob_get_clean();
    }
}



function my_filter_plugin_updates( $value ) {
   if( isset( $value->response['user-registration/user-registration.php'] ) ) {        
      unset( $value->response['user-registration/user-registration.php'] );
    }
    if( isset( $value->response['pagerestrict/pagerestrict.php'] ) ) {        
      unset( $value->response['pagerestrict/pagerestrict.php'] );
    }
    
    return $value;
 }
 add_filter( 'site_transient_update_plugins', 'my_filter_plugin_updates' );
//add_filter('site_transient_update_plugins', '__return_false');



//How to use
